@@ -50,8 +50,11 @@ extensions = [
     "autoapi.extension",
     "sphinx.ext.githubpages",
     "myst_nb",
+    "sphinx.ext.intersphinx",
+    "sphinx_codeautolink",
 ]
 
+myst_heading_anchors = 3
 myst_enable_extensions = [
     "colon_fence",
     "amsmath",
@@ -60,22 +63,40 @@ myst_enable_extensions = [
 templates_path = ["_templates"]
 exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]
 
-# Source code links
+# Codeautolink and intersphinx
+# > make code examples clickable, linking to the docs
+codeautolink_concat_default = True
+intersphinx_mapping = {
+    #'python': ('https://docs.python.org/3', None),
+    "matplotlib": ("https://matplotlib.org/stable", None),
+    "numpy": ("https://numpy.org/doc/stable/", None),
+    #'pandas': ('https://pandas.pydata.org/pandas-docs/stable/', None),
+    #'scipy': ('https://docs.scipy.org/doc/scipy/', None),
+    "xsuite": ("https://xsuite.readthedocs.io/en/latest/", None),
+}
+
+# Linkcode
+# > in API reference, add links to source code on GitHub
 def linkcode_resolve(domain, info):
     if domain != "py" or not info["module"]:
         return None
-    filename = info["module"].replace(".", "/") + ".py"
-    url = f"https://github.com/eltos/xplt/blob/main/{filename}"
     try:
-        # determine line number range
+        # inspect source
         obj = importlib.import_module(info["module"])
         for name in info["fullname"].split("."):
             obj = getattr(obj, name)
+        sourcefile = inspect.getsourcefile(obj)
         sourcecode, line = inspect.getsourcelines(obj)
-        url += f"#L{line}-L{line + len(sourcecode) - 1}"
+        # build link
+        root = "xplt" + os.path.sep
+        if root not in sourcefile:
+            return None  # external source
+        else:
+            path = (root + sourcefile.split(root)[-1]).replace(os.path.sep, "/")
+            filename = f"{path}#L{line}-L{line + len(sourcecode) - 1}"
+            return f"https://github.com/eltos/xplt/blob/main/{filename}"
     except:
-        pass  # e.g. a primitive
-    return url
+        return None
 
 
 # Example notebooks
