@@ -9,18 +9,21 @@ __author__ = "Philipp Niedermayer"
 __contact__ = "eltos@outlook.de"
 __date__ = "2022-09-06"
 
+import types
 
 import matplotlib as mpl
 import numpy as np
 from matplotlib.patches import Ellipse
+from mpl_toolkits.axes_grid1.inset_locator import inset_axes
 
-from .base import XParticlePlot, get, defaults, FixedLimits
-from .util import normalized_coordinates, denormalized_coordinates
+from .base import XPlot, XManifoldPlot, AngleLocator, RadiansFormatter
+from .particles import ParticlePlotMixin
+from .util import get, defaults, normalized_coordinates, denormalized_coordinates
 
 pairwise = np.c_
 
 
-class PhaseSpacePlot(XParticlePlot):
+class PhaseSpacePlot(XPlot, ParticlePlotMixin):
     def __init__(
         self,
         particles=None,
@@ -29,14 +32,11 @@ class PhaseSpacePlot(XParticlePlot):
         *,
         scatter_kwargs=None,
         hist_kwargs=None,
-        ax=None,
         mask=None,
         masks=None,
-        display_units=None,
-        data_units=None,
-        twiss=None,
         color=None,
         cmap="magma_r" or "Blues",
+        cbar_loc=None,
         projections="auto",
         projections_kwargs=None,
         mean=False,
@@ -45,72 +45,49 @@ class PhaseSpacePlot(XParticlePlot):
         std_kwargs=None,
         percentiles=None,
         percentile_kwargs=None,
-        grid=None,
+        nrows=None,
+        ncols=None,
         titles="auto",
-        wrap_zeta=None,
-        beta=None,
-        frev=None,
-        circumference=None,
         animated=False,
-        **subplots_kwargs,
+        twiss=None,
+        **kwargs,
     ):
         """
         A plot for phase space distributions
 
         Args:
-            particles: A dictionary with particle information
-            kind: Defines the properties to plot.
+            particles (Any): A dictionary with particle information
+            kind (str | list): Defines the properties to plot.
                      This can be a nested list or a separated string or a mixture of lists and strings where
                      the first list level (or separator ``,``) determines the subplots,
                      and the second list level (or separator ``-``) determines coordinate pairs.
                      In addition, abbreviations for x-y-parameter pairs are supported (e.g. 'x' for 'x-px').
                      For normalized coordinates, use uppercase letters (e.g. 'X' for 'X-Px').
-
-                     Examples:
-                      - ``'x'``: single subplot with x-px phase space
-                      - ``[['x', 'px']]``: same as above
-                      - ``'x,x-y'``: two suplots the first with x-px and the second with x-y phase space
-                      - ``[['x', 'px'], ['x', 'y']]``: same as above
-
-            plot: Defines the type of plot. Can be 'auto', 'scatter' or 'hist'. Default is 'auto' for which the plot type is chosen automatically based on the number of particles.
-            scatter_kwargs: Additional kwargs for scatter plot
-            hist_kwargs: Additional kwargs for hexbin histogram plot
-            ax: A list of axes to plot onto, length must match the number of subplots. If None, a new figure is created.
-            mask: An index mask to select particles to plot. If None, all particles are plotted.
-            masks: List of masks for each subplot.
-            display_units (dict, optional): Units to display the data in. If None, the units are determined from the data.
-            data_units (dict, optional): Units of the data. If None, the units are determined from the data.
-            twiss (dict, optional): Twiss parameters (alfx, alfy, betx and bety) to use for conversion to normalized phase space coordinates.
-            color (str or list of str): Properties defining the color of points for the scatter plot(s). Implies plot='scatter'. Pass a list of properties to use different values for each subplot
-            cmap: Colormap to use for the hist plot.
-            projections: Add histogrammed projections onto axis. Can be True, False, "x", "y", "auto" or a list of these for each subplot
-            projections_kwargs: Additional kwargs for histogram projection (step plot)
-            mean: Whether to indicate mean of distribution with a cross marker. Boolean or list of booleans for each subplot.
-            mean_kwargs: Additional kwargs for mean cross
-            std: Whether to indicate standard deviation of distribution with an ellipse. Boolean or list of booleans for each subplot.
-            std_kwargs: Additional kwargs for std ellipses.
-            percentiles: List of percentiles (in percent) to indicate in the distribution with ellipses. Can also be a list of lists for each subplot.
-            percentile_kwargs: Additional kwargs for percentile ellipses.
-            grid: Tuple (ncol, nrow) for subplot layout. If None, the layout is determined automatically.
-            titles: List of titles for each subplot or 'auto' to automatically set titles based on plot kind.
-            wrap_zeta: If set, wrap the zeta-coordinate plotted at the machine circumference. Either pass the circumference directly or set this to True to use the circumference from twiss.
-            beta (float, optional): Relativistic beta of particles. Defaults to particles.beta0.
-            frev (float, optional): Revolution frequency of circular line for calculation of particle time.
-            circumference (float, optional): Path length of circular line if frev is not given.
-            animated: If True, improve plotting performance for creating an animation.
-            subplots_kwargs: Keyword arguments passed to matplotlib.pyplot.subplots command when a new figure is created.
+            plot (str): Defines the type of plot. Can be 'auto', 'scatter' or 'hist'. Default is 'auto' for which the plot type is chosen automatically based on the number of particles.
+            scatter_kwargs (dict): Additional kwargs for scatter plot
+            hist_kwargs (dist): Additional kwargs for hexbin histogram plot
+            mask (Any): An index mask to select particles to plot. If None, all particles are plotted.
+            masks (list[mask]): List of masks for each subplot.
+            color (str | list[str]): Properties defining the color of points for the scatter plot(s). Implies plot='scatter'. Pass a list of properties to use different values for each subplot
+            cmap (str): Colormap to use for the hist plot.
+            cbar_loc (str): Location of the colorbar, such as 'right', 'inside upper right', etc.
+            projections (bool | str | list): Add histogrammed projections onto axis. Can be True, False, "x", "y", "auto" or a list of these for each subplot
+            projections_kwargs (dict): Additional kwargs for histogram projection (step plot)
+            mean (bool | list): Whether to indicate mean of distribution with a cross marker. Boolean or list of booleans for each subplot.
+            mean_kwargs (dict): Additional kwargs for mean cross
+            std (bool | list): Whether to indicate standard deviation of distribution with an ellipse. Boolean or list of booleans for each subplot.
+            std_kwargs (dict): Additional kwargs for std ellipses.
+            percentiles (list): List of percentiles (in percent) to indicate in the distribution with ellipses. Can also be a list of lists for each subplot.
+            percentile_kwargs (dict): Additional kwargs for percentile ellipses.
+            ncols (int | None): Number of columns in subplot layout. If None, the layout is determined automatically.
+            nrows (int | None): Number of columns in subplot layout. If None, the layout is determined automatically.
+            titles (list[str]): List of titles for each subplot or 'auto' to automatically set titles based on plot kind.
+            animated (bool): If True, improve plotting performance for creating an animation.
+            twiss (dict | None): Twiss parameters (alfx, alfy, betx and bety) to use for conversion to normalized phase space coordinates.
+            kwargs: See :class:`~.particles.ParticlePlotMixin` and :class:`~.base.XPlot` for additional arguments
 
 
         """
-        super().__init__(
-            data_units=data_units,
-            display_units=display_units,
-            twiss=twiss,
-            beta=beta,
-            frev=frev,
-            circumference=circumference,
-            wrap_zeta=wrap_zeta,
-        )
 
         # parse kind string by splitting at commas and dashes and replacing abbreviations
         abbreviations = dict(
@@ -122,8 +99,8 @@ class PhaseSpacePlot(XParticlePlot):
         )
         if kind is None:
             kind = "x,y,z" if twiss is None else "X,Y,z"
-        self.kind = self._parse_nested_list_string(
-            self._parse_nested_list_string(kind, ",", abbreviations),
+        self.kind = XManifoldPlot.parse_nested_list_string(
+            XManifoldPlot.parse_nested_list_string(kind, ",", abbreviations),
             ",-",  # no abbreviations on second level!
         )
         if np.any([len(k) != 2 for k in self.kind]):
@@ -141,44 +118,60 @@ class PhaseSpacePlot(XParticlePlot):
             projections = n * [projections]
 
         if len(mean) != n:
-            raise ValueError(f"mean must be a boolean or a list of length {n}")
+            raise ValueError(f"mean must be a boolean or a list of length {n}, but got {mean!r}")
         if len(std) != n:
-            raise ValueError(f"std must be a boolean or a list of length {n}")
+            raise ValueError(f"std must be a boolean or a list of length {n}, but got {std!r}")
         if len(percentiles) != n:
-            raise ValueError(f"percentiles list must be flat or of length {n}")
-        if grid and (len(grid) != 2 or grid[0] * grid[1] < n):
-            raise ValueError(f"grid must be a tuple (n, m) with n*m >= {n}")
+            raise ValueError(
+                f"percentiles list must be flat or of length {n}, but got {percentiles!r}"
+            )
         if plot not in ["auto", "scatter", "hist"]:
-            raise ValueError("plot must be 'auto', 'scatter' or 'hist'")
+            raise ValueError(f"plot must be 'auto', 'scatter' or 'hist', but got {plot!r}")
         if color is not None:
             if plot not in ["auto", "scatter"]:
-                raise ValueError(f"Setting color requires plot='scatter', but plot was {plot}")
+                raise ValueError(f"Setting color requires plot='scatter', but got plot={plot!r}")
             plot = "scatter"
             if isinstance(color, str):
                 color = np.resize(color.split(","), n)
         else:
             color = n * [None]
+        if cbar_loc is None:
+            cbar_loc = (
+                "right" if len(color) <= 1 or np.all(color == color[0]) else "inside upper right"
+            )
 
         self.plot = plot
         self.color = color
         self.percentiles = percentiles
         self.projections = projections
-        self.twiss = twiss
-        self.wrap_zeta = wrap_zeta
 
         # Create plot axes
 
         # initialize figure with n subplots
-        if grid:
-            ncol, nrow = grid
-        else:
-            nrow = int(np.sqrt(n))
-            while n % nrow != 0:
-                nrow -= 1
-            ncol = n // nrow
+        if ncols is None:
+            if nrows is None:
+                nrows = int(np.sqrt(n))
+                while n % nrows != 0:
+                    nrows -= 1
+            ncols = n // nrows
+        if ncols * nrows != n:
+            raise ValueError(
+                f"Layout with {ncols} columns and {nrows} rows conflicts with {n} plots!"
+            )
         nntwins = [0 for _ in self.kind]
-        kwargs = defaults(subplots_kwargs, figsize=(4 * ncol, 4 * nrow))
-        self._init_axes(ax, nrow, ncol, nntwins, grid, **kwargs)
+        kwargs = defaults(kwargs, figsize=(4 * ncols, 4 * nrows))
+
+        kwargs = self._init_particle_mixin(
+            twiss=twiss,
+            **kwargs,
+        )
+        super().__init__(
+            nrows=nrows,
+            ncols=ncols,
+            nntwins=nntwins,
+            **kwargs,
+        )
+
         if len(self.axflat) < n:
             raise ValueError(f"Need {n} axes but got only {len(self.axflat)}")
 
@@ -194,16 +187,38 @@ class PhaseSpacePlot(XParticlePlot):
 
         for i, ((a, b), c, ax) in enumerate(zip(self.kind, self.color, self.axflat)):
 
+            # we handle autoscaling manually
+            ax.set_autoscale_on(False)
+
             # 2D phase space distribution
             ##############################
 
             # scatter plot
             kwargs = defaults(scatter_kwargs, s=4, cmap=cmap, lw=0, animated=animated)
             scatter_cmap = kwargs.pop("cmap")  # bypass UserWarning: ignored
+            vmin, vmax = kwargs.pop("vmin", None), kwargs.pop("vmax", None)
             self.artists_scatter[i] = ax.scatter([], [], **kwargs)
+            # TODO: if color is None, use ax.plot([], [], marker=".", ls="") since it is faster
             self.artists_scatter[i].cmap = mpl.colormaps[scatter_cmap]
+            self.artists_scatter[i].vmin_vmax = vmin, vmax
+            # add colorbar
             if c is not None and (np.any(self.color != c) or i == n - 1):
-                self.fig.colorbar(self.artists_scatter[i], ax=ax, label=self.label_for(c))
+                cbargs = dict(label=self.label_for(c))
+                if self.display_unit_for(c) == "rad":
+                    cbargs.update(ticks=AngleLocator(deg=False), format=RadiansFormatter())
+                if cbar_loc.startswith("inside "):
+                    # colorbar inside plot
+                    axins = inset_axes(
+                        ax, width="50%", height="3%", loc=cbar_loc.lstrip("inside ")
+                    )
+                    self.fig.colorbar(
+                        self.artists_scatter[i], **cbargs, cax=axins, orientation="horizontal"
+                    )
+                    axins.tick_params(labelsize=8)
+                    axins.xaxis.offsetText.set_fontsize(8)
+                    axins.xaxis.label.set_size(8)
+                else:
+                    self.fig.colorbar(self.artists_scatter[i], **cbargs, ax=ax, location=cbar_loc)
 
             # hexbin histogram
             self._hxkw = defaults(hist_kwargs, cmap=cmap, rasterized=True, animated=animated)
@@ -217,7 +232,7 @@ class PhaseSpacePlot(XParticlePlot):
             if std[i]:
                 kwargs = defaults(std_kwargs, color="k", lw=1, ls="-", zorder=100)
                 self.artists_std[i] = Ellipse(
-                    [0, 0], 0, 0, fill=False, **kwargs, animated=animated
+                    (0, 0), 0, 0, fill=False, **kwargs, animated=animated
                 )
                 ax.add_artist(self.artists_std[i])
 
@@ -232,7 +247,7 @@ class PhaseSpacePlot(XParticlePlot):
                         ls=(0, [5, 5] + [1, 5] * j),
                         zorder=100,
                     )
-                    artist = Ellipse([0, 0], 0, 0, fill=False, **kwargs, animated=animated)
+                    artist = Ellipse((0, 0), 0, 0, fill=False, **kwargs, animated=animated)
                     ax.add_artist(artist)
                     self.artists_percentiles[i].append(artist)
 
@@ -244,7 +259,13 @@ class PhaseSpacePlot(XParticlePlot):
             else:
                 title = titles[i]
             ax.set(title=title, xlabel=self.label_for(a), ylabel=self.label_for(b))
-            ax.grid(alpha=0.5)
+
+            # Set angle ticks
+            for ab, axis in ((a, ax.xaxis), (b, ax.yaxis)):
+                if self.display_unit_for(ab) == "rad":
+                    self._set_axis_ticks_angle(axis, minor=True, deg=False)
+                elif self.display_unit_for(a) in ("deg", "°"):
+                    self._set_axis_ticks_angle(axis, minor=True, deg=True)
 
             # 1D histogram projections
             ###########################
@@ -269,10 +290,10 @@ class PhaseSpacePlot(XParticlePlot):
         Update the data this plot shows
 
         Args:
-            particles: A dictionary with particle information
-            mask: An index mask to select particles to plot. If None, all particles are plotted.
-            masks: List of masks for each subplot.
-            autoscale: Whether or not to perform autoscaling on all axes.
+            particles (Any): A dictionary with particle information
+            mask (Any): An index mask to select particles to plot. If None, all particles are plotted.
+            masks (list): List of masks for each subplot.
+            autoscale (bool): Whether or not to perform autoscaling on all axes.
 
         Returns:
             List of changed artists.
@@ -287,7 +308,6 @@ class PhaseSpacePlot(XParticlePlot):
         changed_artists = []
 
         for i, ((a, b), c, ax) in enumerate(zip(self.kind, self.color, self.axflat)):
-            ax.autoscale(autoscale)
 
             # coordinates
             x = self.factor_for(a) * self._get_masked(particles, a, masks[i])
@@ -295,10 +315,9 @@ class PhaseSpacePlot(XParticlePlot):
 
             # statistics
             XY = np.array((x, y))
+
             XY0 = np.mean(XY, axis=1)
             UV = XY - XY0[:, np.newaxis]  # centered coordinates
-            if self.artists_std[i] or self.artists_percentiles[i]:
-                evals, evecs = np.linalg.eig(np.cov(UV))  # eigenvalues and -vectors
 
             # 2D phase space distribution
             ##############################
@@ -315,8 +334,23 @@ class PhaseSpacePlot(XParticlePlot):
                 if c is not None:
                     v = self.factor_for(c) * self._get_masked(particles, c, masks[i])
                     if autoscale:
-                        # scatter.set_clim(np.min(v), np.max(v)) # does not always work (bug?)
-                        scatter.norm = mpl.colors.Normalize(np.min(v), np.max(v))
+                        # scatter.set_clim(np.min(v), np.max(v)) # sometimes leaves behind black dots (bug?)
+                        # scatter.norm = mpl.colors.Normalize(np.min(v), np.max(v)) # works, but resets colorbar locator/formatter
+                        # scatter.norm.autoscale(v) # also sometimes leaves behind black dots (bug?)
+
+                        if scatter.colorbar is not None:
+                            locator, formatter = (
+                                scatter.colorbar.locator,
+                                scatter.colorbar.formatter,
+                            )
+                        vmin, vmax = scatter.vmin_vmax
+                        scatter.norm = mpl.colors.Normalize(vmin or np.min(v), vmax or np.max(v))
+                        if scatter.colorbar is not None:
+                            scatter.colorbar.locator, scatter.colorbar.formatter = (
+                                locator,
+                                formatter,
+                            )
+
                     cm = mpl.cm.ScalarMappable(norm=scatter.norm, cmap=scatter.cmap)
                     scatter.set_facecolor(cm.to_rgba(v))
 
@@ -338,37 +372,48 @@ class PhaseSpacePlot(XParticlePlot):
                 self.artists_hexbin[i] = [hexbin_bg, hexbin_fg]
                 changed_artists.extend(self.artists_hexbin[i])
 
-            # 2D mean indicator
+            # 2D mean indicator (cross)
             if self.artists_mean[i]:
                 self.artists_mean[i].set_data(XY0)
                 changed_artists.append(self.artists_mean[i])
 
-            # 2D std indicator
-            if self.artists_std[i]:
-                w, h = 2 * np.sqrt(evals)
-                self.artists_std[i].set(
-                    center=XY0,
-                    width=w,
-                    height=h,
-                    angle=np.degrees(np.arctan2(*evecs[1])),
-                )
-                changed_artists.append(self.artists_std[i])
+            # 2D size indicator (ellipses)
+            if UV.shape[1] > 1 and (self.artists_std[i] or self.artists_percentiles[i]):
+                evals, evecs = np.linalg.eig(np.cov(UV))  # eigenvalues and -vectors
 
-            # 2D percentile indicator
-            if self.artists_percentiles[i]:
-                # normalize distribution using eigenvalues and -vectors
-                NN = np.dot(evecs.T, UV) / np.sqrt(evals)[:, np.newaxis]
-                for j, p in enumerate(self.percentiles[i]):
-                    # percentile in normalized distribution
-                    e = np.percentile(np.sum(NN**2, axis=0), p) ** 0.5
-                    w, h = 2 * e * np.sqrt(evals)
-                    self.artists_percentiles[i][j].set(
+                # 2D std indicator
+                if self.artists_std[i]:
+                    w, h = 2 * np.sqrt(evals)
+                    self.artists_std[i].set(
                         center=XY0,
                         width=w,
                         height=h,
                         angle=np.degrees(np.arctan2(*evecs[1])),
                     )
-                    changed_artists.append(self.artists_percentiles[i][j])
+                    changed_artists.append(self.artists_std[i])
+
+                # 2D percentile indicator
+                if self.artists_percentiles[i]:
+                    # normalize distribution using eigenvalues and -vectors
+                    NN = np.dot(evecs.T, UV) / np.sqrt(evals)[:, np.newaxis]
+                    for j, p in enumerate(self.percentiles[i]):
+                        # percentile in normalized distribution
+                        e = np.percentile(np.sum(NN**2, axis=0), p) ** 0.5
+                        w, h = 2 * e * np.sqrt(evals)
+                        self.artists_percentiles[i][j].set(
+                            center=XY0,
+                            width=w,
+                            height=h,
+                            angle=np.degrees(np.arctan2(*evecs[1])),
+                        )
+                        changed_artists.append(self.artists_percentiles[i][j])
+
+            # Autoscale
+            if autoscale:
+                if plot == "scatter":
+                    self._autoscale(ax, [self.artists_scatter[i]])
+                elif plot == "hist":
+                    self._autoscale(ax, self.artists_hexbin[i])
 
             # 1D histogram projections
             ###########################
@@ -383,10 +428,9 @@ class PhaseSpacePlot(XParticlePlot):
                     hist.set_visible(project)
                     if project:
                         # 1D histogram
-                        counts, edges = np.histogram(v, bins=101)
+                        rng = getattr(ax, f"get_{xy}lim")()
+                        counts, edges = np.histogram(v, bins=101, range=rng)
                         counts = counts / len(v)
-                        # counts = np.bincount((101 * (v - np.min(v)) / (np.max(v) - np.min(v))).astype(int))[:101] / len(v)
-                        # edges = np.linspace(np.min(v), np.max(v), 102)
 
                         steps = (
                             np.append(edges, edges[-1]),
@@ -395,19 +439,6 @@ class PhaseSpacePlot(XParticlePlot):
                         hist.set_data(steps if xy == "x" else steps[::-1])
                         changed_artists.append(hist)
 
-            if autoscale:
-                # ax.relim()  # At present, relim does not support collection instances.
-                if plot == "scatter":
-                    artists = [self.artists_scatter[i]]
-                elif plot == "hist":
-                    artists = self.artists_hexbin[i]
-                else:
-                    artists = []
-                ax.update_datalim(
-                    mpl.transforms.Bbox.union([a.get_datalim(ax.transData) for a in artists])
-                )
-                ax.autoscale(True)
-
         return changed_artists
 
     def title_for(self, a, b):
@@ -422,16 +453,18 @@ class PhaseSpacePlot(XParticlePlot):
             "Y-Py": "Normalized vertical phase space",
             "x-y": "Transverse profile",
         }
+        # TODO: fallback to f"{self.label_for(a, unit=False)}-{self.label_for(b, unit=False)} phase space"
         return titles.get(f"{a}-{b}", titles.get(f"{b}-{a}", f"{a}-{b} phase space"))
 
-    def axline(self, kind, val, also_on_normalized=True, subplots="all", **kwargs):
+    def axline(self, kind, val, *, subplots="all", also_on_normalized=False, delta=0, **kwargs):
         """Plot a vertical or horizontal line for a given coordinate
 
         Args:
             kind (str): Phase space coordinate
             val (float): Value of phase space coordinate
-            also_on_normalized (bool, optional): If true, also plot line for related (de-)normalized phase space coordinate
             subplots (list of int): Subplots to plot line onto. Defaults to all with matching coordinates.
+            also_on_normalized (bool, optional): If true, also plot line for related (de-)normalized phase space coordinates.
+            delta (float, optional): The momentrum error used to convert to (de-)normalized  phase space coordinates.
             kwargs: Arguments for axvline or axhline
 
         """
@@ -447,17 +480,17 @@ class PhaseSpacePlot(XParticlePlot):
                     # same axis found, draw line
                     line(val * self.factor_for(kind), **kwargs)
 
-                elif self.twiss is not None and kind.lower() == a.lower():
+                elif also_on_normalized and self.twiss is not None and kind.lower() == a.lower():
                     # related (de-)normalized axis found, transform and draw line
                     xy = kind.lower()[-1]
                     if kind in "x,y":  # x,y -> X,Y
-                        v = normalized_coordinates(val, 0, self.twiss, xy)[0]
+                        v = normalized_coordinates(val, 0, self.twiss, xy, delta)[0]
                     elif kind in "X,Y":  # X,Y -> x,y
-                        v = denormalized_coordinates(val, 0, self.twiss, xy)[0]
+                        v = denormalized_coordinates(val, 0, self.twiss, xy, delta)[0]
                     elif kind in "px,py":  # px,py -> Px,Py
-                        v = normalized_coordinates(0, val, self.twiss, xy)[1]
+                        v = normalized_coordinates(0, val, self.twiss, xy, delta)[1]
                     elif kind in "Px,Py":  # Px,Py -> px,py
-                        v = denormalized_coordinates(0, val, self.twiss, xy)[1]
+                        v = denormalized_coordinates(0, val, self.twiss, xy, delta)[1]
                     else:
                         continue
                     line(v * self.factor_for(a), **kwargs)
@@ -515,10 +548,10 @@ class PhaseSpacePlot(XParticlePlot):
             if self.twiss is None:
                 raise ValueError("No twiss parameters provided during plot creation")
             alfx, betx, mux, x, px = [
-                get(self.twiss, pre + xy).squeeze() for pre in ["alf", "bet", "mu", "", "p"]
+                get(self.twiss, pre + xy) for pre in ["alf", "bet", "mu", "", "p"]
             ]
-            q = get(self.twiss, "q" + xy).squeeze()
-            q = q + delta * get(self.twiss, "dq" + xy).squeeze()
+            q = get(self.twiss, "q" + xy)
+            q = q + delta * get(self.twiss, "dq" + xy)
 
             # distance d to nearby 3rd order resonance r
             r = round(3 * q) / 3
@@ -543,42 +576,41 @@ class PhaseSpacePlot(XParticlePlot):
                 return XY
 
             lim = ax.dataLim
-            with FixedLimits(ax):
 
-                # plot separatrix
-                if separatrix:
-                    kwarg = defaults(separatrix_kwargs, color="r", ls="--", label="Separatrix")
+            # plot separatrix
+            if separatrix:
+                kwarg = defaults(separatrix_kwargs, color="r", ls="--", label="Separatrix")
 
-                    t = extend
-                    for X, Y in (  # triangle edges
-                        [(-2, -2), (-2 * t, 2 * t)],
-                        [(1 - 3 * t, 1 + 3 * t), (1 + t, 1 - t)],
-                        [(1 - 3 * t, 1 + 3 * t), (-1 - t, -1 + t)],
-                    ):
-                        X = h * np.array(X) / 2
-                        Y = h * 3**0.5 * np.array(Y) / 2
-                        ax.plot(*transform((X, Y)), **kwarg)
-                        kwarg.pop("label", None)
+                t = extend
+                for X, Y in (  # triangle edges
+                    [(-2, -2), (-2 * t, 2 * t)],
+                    [(1 - 3 * t, 1 + 3 * t), (1 + t, 1 - t)],
+                    [(1 - 3 * t, 1 + 3 * t), (-1 - t, -1 + t)],
+                ):
+                    X = h * np.array(X) / 2
+                    Y = h * 3**0.5 * np.array(Y) / 2
+                    ax.plot(*transform((X, Y)), **kwarg)
+                    kwarg.pop("label", None)
 
-                # plot equipotential lines
-                if equipotentials:
-                    kwargs = defaults(
-                        equipotentials_kwargs,
-                        colors="lightgray",
-                        linewidths=1,
-                        alpha=0.5,
-                    )
+            # plot equipotential lines
+            if equipotentials:
+                kwargs = defaults(
+                    equipotentials_kwargs,
+                    colors="lightgray",
+                    linewidths=1,
+                    alpha=0.5,
+                )
 
-                    # X = h * np.linspace(-5, 5, 500)
-                    X = extend * h * np.linspace(-1, 2, 500)
-                    Y = extend * h * 3**0.5 * np.linspace(-1, 1, 500)
-                    X, Y = np.meshgrid(X, Y)
-                    H = (3 * h * (X**2 + Y**2) + 3 * X * Y**2 - X**3) / h**3 / 4
-                    levels = np.linspace(0, extend, int(10 * extend)) ** 2
+                # Hamiltonian normalized to value at separatrix
+                X = extend * h * np.linspace(-1, 2, 500)
+                Y = extend * h * 3**0.5 * np.linspace(-1, 1, 500)
+                X, Y = np.meshgrid(X, Y)
+                H = (3 * h * (X**2 + Y**2) + 3 * X * Y**2 - X**3) / h**3 / 4
+                levels = np.linspace(0, extend, int(10 * extend)) ** 2
 
-                    ax.contour(*transform((X, Y)), H, levels=levels, **kwargs)
+                ax.contour(*transform((X, Y)), H, levels=levels, **kwargs)
 
-                    ax.grid(False)
+                ax.grid(False)
 
             # autoscale to separatrix
             if autoscale:
@@ -586,7 +618,12 @@ class PhaseSpacePlot(XParticlePlot):
                 X = np.array((-2, -2, 1 - 3 * t, 1 + 3 * t, 1 - 3 * t, 1 + 3 * t)) / 2
                 Y = np.array((-2 * t, 2 * t, 1 + t, 1 - t, -1 - t, -1 + t)) * 3**0.5 / 2
                 x, y = transform((h * X, h * Y))
-                ax.dataLim = lim.union(
-                    [mpl.transforms.Bbox([[min(x), min(y)], [max(x), max(y)]])]
-                )
-                ax.autoscale()
+                self._autoscale(ax, data=np.transpose((x, y)), reset=True)
+
+
+## Restrict star imports to local namespace
+__all__ = [
+    name
+    for name, thing in globals().items()
+    if not (name.startswith("_") or isinstance(thing, types.ModuleType))
+]
