@@ -9,101 +9,88 @@ __author__ = "Philipp Niedermayer"
 __contact__ = "eltos@outlook.de"
 __date__ = "2022-11-08"
 
+import types
 
-from .base import XPlot, defaults
+from .util import defaults
+from .base import XManifoldPlot
 from .line import KnlPlot
 
 
-class TwissPlot(XPlot):
+class TwissPlot(XManifoldPlot):
     def __init__(
         self,
         twiss=None,
         kind="bet-dx,x+y",
         *,
-        ax=None,
         line=None,
-        line_kwargs=dict(),
-        display_units=None,
-        **subplots_kwargs,
+        line_kwargs={},
+        **kwargs,
     ):
         """
         A plot for twiss parameters and closed orbit
 
-        :param kind: Defines the properties to plot.
-                     This can be a nested list or a separated string or a mixture of lists and strings where
-                     the first list level (or separator ``,``) determines the subplots,
-                     the second list level (or separator ``-``) determines any twinx-axes,
-                     and the third list level (or separator ``+``) determines plots on the same axis.
-                     In addition, abbreviations for x-y-parameter pairs are supported (e.g. 'bet' for 'betx+bety').
+        Args:
+            twiss (Any): Dictionary with twiss information
+            kind (str | list): Defines the properties to plot.
+                This is a manifold subplot specification string like ``"bet-dx,x+y"``, see :class:`~.base.XManifoldPlot` for details.
+                In addition, abbreviations for x-y-parameter pairs are supported (e.g. 'bet' for 'betx+bety').
+            line (xtrack.Line): Line of elements. If given, adds a line plot to the top.
+            line_kwargs (dict): Keyword arguments passed to line plot.
+            kwargs: See :class:`~.base.XPlot` for additional arguments
 
-                     Examples:
-                      - ``'bet-dx'``: single subplot with 'betx' and 'bety' on the left and 'dx' on the right axis
-                      - ``[[['betx', 'bety'], ['dx']]]``: same as above
-                      - ``'betx+alf,mu'``: two suplots the first with 'betx', 'alfx' and 'alfy' and the second with 'mux' and 'muy'
-                      - ``[[['betx', 'alfx', 'alfy']], [['mux', 'muy']]]``: same as above
-
-        :param twiss: Dictionary with twiss information
-        :param ax: A list of axes to plot onto, length must match the number of subplots and optional line plot. If None, a new figure is created.
-                   If required, twinx-axes will be added automatically.
-        :param line: Line of elements. If given, adds a line plot to the top.
-        :param line_kwargs: Keyword arguments passed to line plot.
-        :param display_units: Dictionary with units for parameters. Supports prefix notation, e.g. 'bet' for 'betx' and 'bety'.
-        :param subplots_kwargs: Keyword arguments passed to matplotlib.pyplot.subplots command when a new figure is created.
 
         """
+        subs = {p: f"{p}x+{p}y" for p in "alf,bet,gam,mu,d,dp,q,dq".split(",")}
+
+        if line:
+            kind = self.parse_nested_list_string(kind, subs=subs)
+            kind = [[[None]], *kind]
+
         super().__init__(
-            display_units=dict(
-                dict(s="m", x="mm", y="mm", p="mrad", bet="m", d="m"),
-                **(display_units or {}),
-            )
+            on_x="s",
+            on_y=kind,
+            on_y_subs=subs,
+            display_units=defaults(kwargs.pop("display_units", None), bet="m", d="m"),
+            **kwargs,
         )
 
-        # parse kind string
-        subs = {a: f"{a}x+{a}y" for a in ("", "p", "alf", "bet", "gam", "mu", "d", "dp")}
-        self.kind = self._parse_nested_list_string(kind, ",-+", subs)
+        # create plot elements
+        def create_artists(i, j, k, a, p):
+            if line and i == 0:
+                return None  # skip line plot placeholder
+            return a.plot([], [], label=self._legend_label_for(p))[0]
 
-        # initialize figure with n subplots
-        n, nntwin = len(self.kind), [len(tw) - 1 for tw in self.kind]
-        if line:
-            n += 1
-            nntwin = [0] + nntwin
-        self._init_axes(ax, n, 1, nntwin, sharex="col", **subplots_kwargs)
+        self._create_artists(create_artists)
 
         # Create line plot
         self.lineplot = None
         if line:
-            self.lineplot = KnlPlot(line, ax=self.axis_for(0), **line_kwargs)
+            self.lineplot = KnlPlot(line, ax=self.axis(0), **line_kwargs)
             self.lineplot.ax.set(xlabel=None)
 
-        # Format plot axes
-        self.axis_for(-1).set(xlabel=self.label_for("s"))
-
-        # create plot elements
-        def create_artists(i, j, k, a, p):
-            return a.plot([], [], label=self.label_for(p, unit=False))[0]
-
-        self._init_artists([[], *self.kind] if line else self.kind, create_artists)
-
         # set data
-        if twiss:
+        if twiss is not None:
             self.update(twiss, autoscale=True)
 
     def update(self, twiss, *, autoscale=False, line=None):
         """
         Update the twiss data this plot shows
 
-        :param twiss: Dictionary with twiss information.
-        :param line: Line of elements.
-        :param autoscale: Whether or not to perform autoscaling on all axes.
-        :return: changed artists.
+        Args:
+            twiss (Any): Dictionary with twiss information
+            autoscale (bool): Whether or not to perform autoscaling on all axes.
+            line (xtack.Line): Line of elements.
+
+        Returns:
+            changed artists
         """
         s = self.factor_for("s")
         changed = []
-        for i, ppp in enumerate(self.kind):
-            if self.lineplot is not None:
-                i += 1  # skip line plot
+        for i, ppp in enumerate(self.on_y):
+            if self.lineplot is not None and i == 0:
+                continue  # skip line plot
             for j, pp in enumerate(ppp):
-                a = self.axis_for(i, j)
+                a = self.axis(i, j)
                 for k, p in enumerate(pp):
                     f = self.factor_for(p)
                     self.artists[i][j][k].set_data((s * twiss["s"], f * twiss[p]))
@@ -114,7 +101,7 @@ class TwissPlot(XPlot):
                     a.set(xlim=(s * min(twiss["s"]), s * max(twiss["s"])))
 
         if line:
-            self.lineplot.update(line, autoscale=autoscale)
+            changed.append(self.lineplot.update(line, autoscale=autoscale))
 
         return changed
 
@@ -128,7 +115,7 @@ class TwissPlot(XPlot):
             kwargs: Arguments for axvline or axhline
 
         """
-        return self.axspan(kind, val, None, subplots=subplots, **kwargs)
+        self.axspan(kind, val, None, subplots=subplots, **kwargs)
 
     def axspan(self, kind, val, val_to=None, subplots="all", **kwargs):
         """Plot a vertical or horizontal span (or line) for a given coordinate
@@ -161,12 +148,12 @@ class TwissPlot(XPlot):
 
         else:
             # horizontal span or line
-            for i, p_subplot in enumerate(self.kind):
+            for i, p_subplot in enumerate(self.on_y):
                 if subplots != "all" and i not in subplots:
                     continue
 
                 for j, p_axis in enumerate(p_subplot):
-                    a = self.axis_for(i, j)
+                    a = self.axis(i, j)
                     for k, p in enumerate(p_axis):
                         if p == kind:  # axis found
                             if val_to is None:  # only a line
@@ -177,3 +164,11 @@ class TwissPlot(XPlot):
                                     val_to * self.factor_for("s"),
                                     **kwargs,
                                 )
+
+
+## Restrict star imports to local namespace
+__all__ = [
+    name
+    for name, thing in globals().items()
+    if not (name.startswith("_") or isinstance(thing, types.ModuleType))
+]
